import {DeployConfig, ReqDeployConfig} from './types.js'
import {gitInit} from '../../prompts/git-init.js'
import {git, error} from '@shopify/cli-kit'

const MINIMAL_GIT_IGNORE: git.GitIgnoreTemplate = {
  system: ['.DS_Store'],
  logs: ['logs', '*.log', 'npm-debug.log*', 'yarn-debug.log*', 'yarn-error.log*'],
  testing: ['/coverage', '*.lcov'],
  dependencies: ['/node_modules', '.npm', '.yarn-integrity', '/.pnp', '.pnp.js'],
  typescript: ['*.tsbuildinfo'],
  environment: ['.env', '.env.test', '.env.local'],
  production: ['/dist'],
}

export const validateProject = async (config: DeployConfig) => {
  await git.ensurePresentOrAbort()
  try {
    await git.ensureInsideGitDirectory(config.path)
  } catch (err: unknown) {
    if ((err as error.Abort)?.message !== git.OutsideGitDirectoryError(config.path).message) throw error
    await initializeGit(config)
  }
}

export const initializeGit = async (config: DeployConfig) => {
  if (!config.assumeYes) {
    const shouldGitInit = await gitInit()
    if (!shouldGitInit) throw new error.AbortSilent()
  }

  await git.initializeRepository(config.path)
  git.createGitIgnore(config.path, MINIMAL_GIT_IGNORE)
  await git.addAll(config.path)
  await git.commit('Initial commit generated by Hydrogen', {directory: config.path})
}

export const fillDeployConfig = async (config: DeployConfig): Promise<ReqDeployConfig> => {
  const [latestCommit, commitRef] = await Promise.all([
    git.getLatestCommit(config.path),
    git.getHeadSymbolicRef(config.path),
  ])

  return {
    ...config,
    commitMessage: config.commitMessage ?? latestCommit.message,
    commitAuthor: config.commitAuthor ?? latestCommit.author_name,
    commitSha: latestCommit.hash,
    timestamp: latestCommit.date,
    commitRef,
  }
}
